package Algorithm::FloodControl::Backend::Cache::Memcached::Fast;

use utf8;
use strict;
use warnings;
use Algorithm::FloodControl::Backend::Cache::Memcached;
use base 'Algorithm::FloodControl::Backend::Cache::Memcached';

our ($VERSION) = '$Revision: 2609 $' =~ m{ \$Revision: \s+ (\S+) }mx;

1;

__END__

=head1 NAME

Algorithm::FloodControl::Backend::Cache::Memcached::Fast

=head1 DESCRIPTION

Cache::Memcached::Fast backend. See Cache::Memcached backend

=head1 BUGS

=head1 NOTES

=head1 AUTHOR

Andrey Kostenko (), <andrey@kostenko.name>

=head1 COMPANY

Rambler Internet Holding

=head1 CREATED

01.11.2008 14:44:54 MSK

=cut

